# -*- coding: utf-8 -*-
import string

from django.core.exceptions import ValidationError
from django.test import TestCase

from ..validators import validate_password_strength


class PasswordsTest(TestCase):
    too_simple = (
        'Password must have at least ' +
        'one upper case letter, one lower case letter, and one number.'
    )

    too_fancy = (
        'Password only accepts the following symbols ' + string.punctuation
    )

    def test_no_upper(self):
        password = 'aaaa1111'
        with self.assertRaises(ValidationError) as error:
            validate_password_strength(password)
        self.assertEqual(error.exception.message, self.too_simple)

    def test_no_lower(self):
        password = 'AAAA1111'
        with self.assertRaises(ValidationError) as error:
            validate_password_strength(password)

        self.assertEqual(error.exception.message, self.too_simple)

    def test_no_number(self):
        password = 'AAAAaaaa'
        with self.assertRaises(ValidationError) as error:
            validate_password_strength(password)

        self.assertEqual(error.exception.message, self.too_simple)

    def test_symbols(self):
        """Ensure all acceptable symbols are acceptable."""
        for symbol in string.punctuation:
            password = 'AAaa111' + symbol
            self.assertIsNone(validate_password_strength(password))

    def test_non_ascii(self):
        password = u'AA11aa££'  # £ is not an ASCII character.
        with self.assertRaises(ValidationError) as error:
            validate_password_strength(password)

        self.assertEqual(error.exception.message, self.too_fancy)

    def test_ok(self):
        password = 'AAAaaa11'
        self.assertIsNone(validate_password_strength(password))



from django.contrib.auth import get_user_model, signals
from django.contrib.auth.tokens import default_token_generator
from django.core import signing
from django.utils.encoding import force_text
from django.utils.http import urlsafe_base64_decode
from django.utils.translation import ugettext_lazy as _
from rest_framework import generics, response, status, views
from rest_framework.authentication import get_authorization_header
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.exceptions import PermissionDenied
from rest_framework.permissions import AllowAny, IsAuthenticated

from . import exceptions, models, permissions, serializers, throttling


User = get_user_model()


class GetAuthToken(ObtainAuthToken):
    """
    Obtain an authentication token.

    Define a `POST` (create) method to authenticate a user using their `email` and
    `password` and return a `token` if successful.
    The `token` remains valid until `settings.AUTH_TOKEN_MAX_AGE` time has passed.

    `DELETE` method removes the current `token` from the database.
    """
    model = models.AuthToken
    throttle_classes = [
        throttling.UsernameLoginRateThrottle,
        throttling.LoginRateThrottle,
    ]
    throttle_scope = 'logins'

    def post(self, request):
        """Create auth token. Differs from DRF that it always creates new token
        but not re-using them."""
        serializer = self.serializer_class(data=request.DATA)
        if serializer.is_valid():
            user = serializer.validated_data['user']
            signals.user_logged_in.send(type(self), user=user, request=request)
            token = self.model.objects.create(user=user)
            token.update_expiry()
            return response.Response({'token': token.key})

        return response.Response(
            serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, *args, **kwargs):
        """Delete auth token when `delete` request was issued."""
        # Logic repeated from DRF because one cannot easily reuse it
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != b'token':
            return response.Response(status=status.HTTP_400_BAD_REQUEST)

        if len(auth) == 1:
            msg = 'Invalid token header. No credentials provided.'
            return response.Response(msg, status=status.HTTP_400_BAD_REQUEST)
        elif len(auth) > 2:
            msg = 'Invalid token header. Token string should not contain spaces.'
            return response.Response(msg, status=status.HTTP_400_BAD_REQUEST)

        try:
            token = self.model.objects.get(key=auth[1])
        except self.model.DoesNotExist:
            pass
        else:
            token.delete()
            signals.user_logged_out.send(
                type(self),
                user=token.user,
                request=request,
            )
        return response.Response(status=status.HTTP_204_NO_CONTENT)


class UserRegister(generics.CreateAPIView):
    """
    Register a new `User`.

    An email to validate the new account is sent if `email_verified`
    is set to `False`.
    """
    serializer_class = serializers.RegistrationSerializer
    permission_classes = [permissions.IsNotAuthenticated]

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid():
            return self.is_valid(serializer)
        return self.is_invalid(serializer)

    def is_invalid(self, serializer):
        return response.Response(
            data=serializer.errors,
            status=status.HTTP_400_BAD_REQUEST,
        )

    def is_valid(self, serializer):
        user = serializer.save()
        if not user.email_verified:
            user.send_validation_email()
            ok_message = _(
                'Your account has been created and an activation link sent ' +
                'to your email address. Please check your email to continue.'
            )
        else:
            ok_message = _('Your account has been created.')

        return response.Response(
            data={'data': ok_message},
            status=status.HTTP_201_CREATED,
        )


class PasswordResetEmail(generics.GenericAPIView):
    """
    Send a password reset email to a user on request.

    A user can request a password request email by providing their email address.
    If the user is not found no error is raised.
    """
    permission_classes = [permissions.IsNotAuthenticated]
    template_name = 'user_management/password_reset_email.html'
    serializer_class = serializers.PasswordResetEmailSerializer
    throttle_classes = [throttling.PasswordResetRateThrottle]
    throttle_scope = 'passwords'

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.DATA)
        if not serializer.is_valid():
            return response.Response(
                serializer.errors,
                status=status.HTTP_400_BAD_REQUEST,
            )

        email = serializer.data['email']
        try:
            user = User.objects.get_by_natural_key(email)
        except User.DoesNotExist:
            pass
        else:
            user.send_password_reset()

        msg = _('Password reset request successful. Please check your email.')
        return response.Response(msg, status=status.HTTP_204_NO_CONTENT)


class OneTimeUseAPIMixin(object):
    """
    Use a `uid` and a `token` to allow one-time access to a view.

    Set user as a class attribute or raise an `InvalidExpiredToken`.
    """
    def initial(self, request, *args, **kwargs):
        uidb64 = kwargs['uidb64']
        uid = urlsafe_base64_decode(force_text(uidb64))

        try:
            self.user = User.objects.get(pk=uid)
        except User.DoesNotExist:
            raise exceptions.InvalidExpiredToken()

        token = kwargs['token']
        if not default_token_generator.check_token(self.user, token):
            raise exceptions.InvalidExpiredToken()

        return super(OneTimeUseAPIMixin, self).initial(
            request,
            *args,
            **kwargs
        )


class PasswordReset(OneTimeUseAPIMixin, generics.UpdateAPIView):
    """
    Reset a user's password.

    This view is generally called when a user has followed an email link to
    reset a password.

    This view will check first if the `uid` and `token` are valid.

    `PasswordReset` is called with an `UPDATE` containing the new password
    (`new_password` and `new_password2`).
    """
    permission_classes = [permissions.IsNotAuthenticated]
    model = User
    serializer_class = serializers.PasswordResetSerializer

    def get_object(self):
        return self.user


class PasswordChange(generics.UpdateAPIView):
    """
    Change a user's password.

    Give ability to `PUT` (update) a password when authenticated by submitting current
    password.
    """
    model = User
    permission_classes = (IsAuthenticated,)
    serializer_class = serializers.PasswordChangeSerializer

    def get_object(self):
        return self.request.user


class VerifyAccountView(views.APIView):
    """
    Verify a new user's email address.

    Verify a newly created account by checking the `uid` and `token` in a `POST` request.
    """
    permission_classes = [AllowAny]
    ok_message = _('Your account has been verified.')

    def initial(self, request, *args, **kwargs):
        """
        Use `token` to allow one-time access to a view.

        Set user as a class attribute or raise an `InvalidExpiredToken`.
        """
        try:
            email_data = signing.loads(kwargs['token'])
        except signing.BadSignature:
            raise exceptions.InvalidExpiredToken

        email = email_data['email']

        try:
            self.user = User.objects.get_by_natural_key(email)
        except User.DoesNotExist:
            raise exceptions.InvalidExpiredToken()

        return super(VerifyAccountView, self).initial(
            request,
            *args,
            **kwargs
        )

    def post(self, request, *args, **kwargs):
        if self.user.email_verified:
            return response.Response(status=status.HTTP_403_FORBIDDEN)

        self.user.email_verified = True
        self.user.is_active = True
        self.user.save()

        return response.Response(
            data={'data': self.ok_message},
            status=status.HTTP_201_CREATED,
        )


class ProfileDetail(generics.RetrieveUpdateDestroyAPIView):
    """
    Allow a user to view and edit their profile information.

    `GET`, `UPDATE` and `DELETE` current logged-in user.
    """
    model = User
    permission_classes = (IsAuthenticated,)
    serializer_class = serializers.ProfileSerializer

    def get_object(self):
        return self.request.user


class UserList(generics.ListCreateAPIView):
    """
    Return information about all users and allow creation of new users.

    Allow to `GET` a list users and to `POST` new user for admin user only.
    """
    queryset = User.objects.all()
    permission_classes = (IsAuthenticated, permissions.IsAdminOrReadOnly)
    serializer_class = serializers.UserSerializerCreate


class UserDetail(generics.RetrieveUpdateDestroyAPIView):
    """
    Display information about a user.

    Allow admin users to update or delete user information.
    """
    queryset = User.objects.all()
    permission_classes = (IsAuthenticated, permissions.IsAdminOrReadOnly)
    serializer_class = serializers.UserSerializer


class ResendConfirmationEmail(generics.GenericAPIView):
    """
    Resend a confirmation email.

    `POST` request to resend a confirmation email for existing user. If user is
    authenticated the email sent should match.
    """
    permission_classes = [AllowAny]
    serializer_class = serializers.ResendConfirmationEmailSerializer
    throttle_classes = [throttling.ResendConfirmationEmailRateThrottle]
    throttle_scope = 'confirmations'

    def initial(self, request, *args, **kwargs):
        """
        Use `token` to allow one-time access to a view.

        Set user as a class attribute or raise an `InvalidExpiredToken`.
        """
        email = request.DATA.get('email')
        if request.user.is_authenticated() and email != request.user.email:
            raise PermissionDenied()

        return super(ResendConfirmationEmail, self).initial(
            request,
            *args,
            **kwargs
        )

    def post(self, request, *args, **kwargs):
        """Validate `email` and send a request to confirm it."""
        serializer = self.serializer_class(data=request.DATA)

        if not serializer.is_valid():
            return response.Response(
                serializer.errors,
                status=status.HTTP_400_BAD_REQUEST,
            )

        serializer.user.send_validation_email()
        msg = _('Email confirmation sent.')
        return response.Response(msg, status=status.HTTP_204_NO_CONTENT)



from django.views.debug import SafeExceptionReporterFilter
from raven.contrib.django.client import DjangoClient


class SensitiveDjangoClient(DjangoClient):
    """
    Hide sensitive request data from being logged by Sentry.

    Borrowed from http://stackoverflow.com/a/23966581/240995
    """
    def get_data_from_request(self, request):
        request.POST = SafeExceptionReporterFilter().get_post_parameters(request)
        result = super(SensitiveDjangoClient, self).get_data_from_request(request)

        # override the request.data with POST data
        # POST data contains no sensitive info in it
        result['request']['data'] = request.POST

        # remove the whole cookie as it contains DRF auth token and session id
        if 'cookies' in result['request']:
            del result['request']['cookies']
        if 'Cookie' in result['request']['headers']:
            del result['request']['headers']['Cookie']

        return result



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt

from .. import views


urlpatterns = patterns(
    '',
    url(
        regex=(
            r'^auth/password_reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/'
            r'(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/?$'
        ),
        view=csrf_exempt(views.PasswordReset.as_view()),
        name='password_reset_confirm',
    ),
    url(
        regex=r'^auth/password_reset/?$',
        view=views.PasswordResetEmail.as_view(),
        name='password_reset',
    ),
)



import incuna_mail
from django.utils.translation import ugettext_lazy as _
from pigeon.notification import Notification


def password_reset_email_context(notification):
    """Email context to reset a user password."""
    return {
        'protocol': 'https',
        'uid': notification.user.generate_uid(),
        'token': notification.user.generate_token(),
        'site': notification.site,
    }


def validation_email_context(notification):
    """Email context to verify a user email."""
    return {
        'protocol': 'https',
        'token': notification.user.generate_validation_token(),
        'site': notification.site,
    }


def email_handler(notification, email_context):
    """Send a notification by email."""
    incuna_mail.send(
        to=notification.user.email,
        subject=notification.email_subject,
        template_name=notification.text_email_template,
        html_template_name=notification.html_email_template,
        context=email_context(notification),
    )


def password_reset_email_handler(notification):
    """Password reset email handler."""
    subject = _('{domain} password reset').format(domain=notification.site.domain)
    notification.email_subject = subject
    email_handler(notification, password_reset_email_context)


def validation_email_handler(notification):
    """Validation email handler."""
    subject = _('{domain} account validate').format(domain=notification.site.domain)
    notification.email_subject = subject
    email_handler(notification, validation_email_context)


class PasswordResetNotification(Notification):
    """`PasswordResetNotification` defines text and html email templates."""
    handlers = (password_reset_email_handler,)
    text_email_template = 'user_management/password_reset_email.txt'
    html_email_template = 'user_management/password_reset_email.html'


class ValidationNotification(Notification):
    """`ValidationNotification` defines text and html email templates."""
    handlers = (validation_email_handler,)
    text_email_template = 'user_management/account_validation_email.txt'
    html_email_template = 'user_management/account_validation_email.html'



from django.core.exceptions import ValidationError
from django.test import TestCase
from incuna_test_utils.compat import Python2AssertMixin

from . factories import UserFactory
from .. import admin_forms


class UserCreationFormTest(Python2AssertMixin, TestCase):
    """Assert UserCreationForm` behaves properly."""
    form = admin_forms.UserCreationForm

    def test_fields(self):
        """Assert `fields`."""
        fields = self.form.base_fields.keys()
        expected = ('email', 'password1', 'password2')
        self.assertCountEqual(fields, expected)

    def test_required_fields(self):
        """Assert required `fields` are correct."""
        form = self.form({})
        expected = 'This field is required.'
        self.assertIn(expected, form.errors['email'])
        self.assertIn(expected, form.errors['password1'])
        self.assertIn(expected, form.errors['password2'])

    def test_clean_email(self):
        email = 'Test@example.com'

        form = self.form()
        form.cleaned_data = {'email': email}

        self.assertEqual(form.clean_email(), email.lower())

    def test_clean_duplicate_email(self):
        user = UserFactory.create()

        form = self.form()
        form.cleaned_data = {'email': user.email}

        with self.assertRaises(ValidationError):
            form.clean_email()

    def test_clean(self):
        data = {'password1': 'pass123', 'password2': 'pass123'}

        form = self.form()
        form.cleaned_data = data

        self.assertEqual(form.clean(), data)

    def test_clean_mismatched(self):
        data = {'password1': 'pass123', 'password2': 'pass321'}

        form = self.form()
        form.cleaned_data = data

        with self.assertRaises(ValidationError):
            form.clean()

    def test_save(self):
        data = {
            'email': 'test@example.com',
            'password1': 'pass123',
            'password2': 'pass123',
        }

        form = self.form(data)
        self.assertTrue(form.is_valid(), form.errors.items())

        user = form.save()
        self.assertEqual(user.email, data['email'])


class UserChangeFormTest(Python2AssertMixin, TestCase):
    """Assert `UserChangeForm` behaves properly."""
    form = admin_forms.UserChangeForm

    def test_fields(self):
        """Assert `fields`."""
        fields = self.form.base_fields.keys()
        expected = (
            'email',
            'groups',
            'is_active',
            'is_staff',
            'is_superuser',
            'last_login',
            'name',
            'password',
            'user_permissions',
        )
        self.assertCountEqual(fields, expected)

    def test_clean_password(self):
        password = 'pass123'
        data = {'password': password}
        user = UserFactory.build()

        form = self.form(data, instance=user)
        self.assertNotEqual(form.clean_password(), password)



from django.test import TestCase
from rest_framework.status import HTTP_400_BAD_REQUEST

from ..exceptions import InvalidExpiredToken


class InvalidExpiredTokenTest(TestCase):
    """Assert `InvalidExpiredToken` behaves as expected."""
    def test_raise(self):
        """Assert `InvalidExpiredToken` can be raised."""
        with self.assertRaises(InvalidExpiredToken) as error:
            raise InvalidExpiredToken
        self.assertEqual(error.exception.status_code, HTTP_400_BAD_REQUEST)
        message = error.exception.detail.format()
        self.assertEqual(message, 'Invalid or expired token.')



from django.contrib.auth.models import BaseUserManager
from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.models import Site
from django.core import checks, signing
from django.db import models
from django.utils import timezone
from django.utils.encoding import force_bytes, python_2_unicode_compatible
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import ugettext_lazy as _

from user_management.utils import notifications


class UserManager(BaseUserManager):
    """Django requires user managers to have create_user & create_superuser."""
    def create_user(self, email, password=None, **extra_fields):
        if not email:
            raise ValueError(_('The given email address must be set'))
        email = self.normalize_email(email).lower()
        user = self.model(
            email=email,
            last_login=timezone.now(),
            **extra_fields
        )
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        fields = {
            'is_staff': True,
            'is_superuser': True,
        }
        fields.update(extra_fields)
        user = self.create_user(email, password, **fields)
        return user

    def get_by_natural_key(self, email):
        """Get user by email with case-insensitive exact match.

        `get_by_natural_key` is used to `authenticate` a user, see:
        https://github.com/django/django/blob/c5780adeecfbd85a80b5aa7130dd86e78b23e497/django/contrib/auth/backends.py#L16
        """
        return self.get(email__iexact=email)


class DateJoinedUserMixin(models.Model):
    date_joined = models.DateTimeField(
        verbose_name=_('date joined'),
        default=timezone.now,
        editable=False,
    )

    class Meta:
        abstract = True


class EmailUserMixin(models.Model):
    email = models.EmailField(
        verbose_name=_('Email address'),
        unique=True,
        max_length=511,
    )
    email_verified = True

    objects = UserManager()

    USERNAME_FIELD = 'email'

    class Meta:
        abstract = True


class IsStaffUserMixin(models.Model):
    is_staff = models.BooleanField(_('staff status'), default=False)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class NameUserMethodsMixin:
    def get_full_name(self):
        return self.name

    def get_short_name(self):
        return self.name

    def __str__(self):
        return self.name


class NameUserMixin(NameUserMethodsMixin, models.Model):
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=255,
    )
    REQUIRED_FIELDS = ['name']

    class Meta:
        abstract = True
        ordering = ['name']


class BasicUserFieldsMixin(
        DateJoinedUserMixin, EmailUserMixin, IsStaffUserMixin, NameUserMixin):
    class Meta:
        abstract = True


class ActiveUserMixin(models.Model):
    is_active = models.BooleanField(_('active'), default=True)

    class Meta:
        abstract = True


class VerifyEmailManager(UserManager):
    def create_superuser(self, email, password, **extra_fields):
        fields = {
            'is_active': True,
        }
        fields.update(extra_fields)
        user = super(VerifyEmailManager, self).create_superuser(
            email,
            password,
            **fields)
        return user


class EmailVerifyUserMethodsMixin:
    """
    Define how validation and password reset emails are sent.

    `password_reset_notification` and `validation_notification` can be overriden to
    provide custom settings to send emails.
    """
    password_reset_notification = notifications.PasswordResetNotification
    validation_notification = notifications.ValidationNotification

    def generate_validation_token(self):
        """Generate user token for account validation."""
        data = {'email': self.email}
        return signing.dumps(data)

    def generate_token(self):
        """Generate user token for password reset."""
        return default_token_generator.make_token(self)

    def generate_uid(self):
        """Generate user uid for password reset."""
        return urlsafe_base64_encode(force_bytes(self.pk))

    def send_validation_email(self):
        """Send a validation email to the user's email address."""
        if self.email_verified:
            raise ValueError(_('Cannot validate already active user.'))

        site = Site.objects.get_current()
        self.validation_notification(user=self, site=site).notify()

    def send_password_reset(self):
        """Send a password reset to the user's email address."""
        site = Site.objects.get_current()
        self.password_reset_notification(user=self, site=site).notify()


class EmailVerifyUserMixin(EmailVerifyUserMethodsMixin, models.Model):
    is_active = models.BooleanField(_('active'), default=False)
    email_verified = models.BooleanField(
        _('Email verified?'),
        default=False,
        help_text=_('Indicates if the email address has been verified.'))

    objects = VerifyEmailManager()

    class Meta:
        abstract = True

    @classmethod
    def check(cls, **kwargs):
        errors = super(EmailVerifyUserMixin, cls).check(**kwargs)
        errors.extend(cls._check_manager(**kwargs))
        return errors

    @classmethod
    def _check_manager(cls, **kwargs):
        if isinstance(cls.objects, VerifyEmailManager):
            return []

        return [
            checks.Warning(
                "Manager should be an instance of 'VerifyEmailManager'",
                hint="Subclass a custom manager from 'VerifyEmailManager'",
                obj=cls,
                id='user_management.W001',
            ),
        ]


class VerifyEmailMixin(EmailVerifyUserMixin, BasicUserFieldsMixin):
    class Meta:
        abstract = True


class AvatarMixin(models.Model):
    avatar = models.ImageField(upload_to='user_avatar', null=True, blank=True)

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
import string

from django.test import TestCase
from rest_framework.fields import Field
from rest_framework.reverse import reverse
from rest_framework.serializers import ValidationError

from user_management.models.tests.factories import UserFactory
from user_management.models.tests.utils import RequestTestCase
from user_management.tests.utils import iso_8601
from .. import serializers


class ProfileSerializerTest(TestCase):
    def test_serialize(self):
        user = UserFactory.build()
        serializer = serializers.ProfileSerializer(user)

        expected = {
            'name': user.name,
            'email': user.email,
            'date_joined': iso_8601(user.date_joined),
        }
        self.assertEqual(serializer.data, expected)

    def test_deserialize(self):
        user = UserFactory.build()
        data = {
            'name': 'New Name',
        }
        serializer = serializers.ProfileSerializer(user, data=data)
        self.assertTrue(serializer.is_valid())


class PasswordChangeSerializerTest(TestCase):
    def test_deserialize_passwords(self):
        old_password = '0ld_passworD'
        new_password = 'n3w_Password'

        user = UserFactory.create(password=old_password)

        serializer = serializers.PasswordChangeSerializer(user, data={
            'old_password': old_password,
            'new_password': new_password,
            'new_password2': new_password,
        })
        self.assertTrue(serializer.is_valid())

        serializer.save()
        self.assertTrue(user.check_password(new_password))

    def test_deserialize_invalid_old_password(self):
        old_password = '0ld_passworD'
        new_password = 'n3w_Password'

        user = UserFactory.build(password=old_password)

        serializer = serializers.PasswordChangeSerializer()
        data = {
            'old_password': 'invalid_password',
            'new_password': new_password,
            'new_password2': new_password,
        }
        with self.assertRaises(ValidationError):
            serializer.update(user, data)

    def test_deserialize_invalid_new_password(self):
        old_password = '0ld_passworD'
        new_password = '2Short'

        user = UserFactory.build(password=old_password)

        serializer = serializers.PasswordChangeSerializer(user, data={
            'old_password': old_password,
            'new_password': new_password,
            'new_password2': new_password,
        })
        self.assertFalse(serializer.is_valid())
        self.assertIn('new_password', serializer.errors)

    def test_deserialize_mismatched_passwords(self):
        old_password = '0ld_passworD'
        new_password = 'n3w_Password'
        other_password = 'other_new_password'

        user = UserFactory.create(password=old_password)

        serializer = serializers.PasswordChangeSerializer(user, data={
            'old_password': old_password,
            'new_password': new_password,
            'new_password2': other_password,
        })
        self.assertFalse(serializer.is_valid())

    def test_no_change_password(self):
        """A password cannot be changed if it doesn't change!"""
        password = 'Same_passw0rd'

        user = UserFactory.create(password=password)

        serializer = serializers.PasswordChangeSerializer(user, data={
            'old_password': password,
            'new_password': password,
            'new_password2': password,
            }
        )
        self.assertFalse(serializer.is_valid())
        self.assertIn('new_password', serializer.errors)


class PasswordResetSerializerTest(TestCase):
    def test_deserialize_passwords(self):
        new_password = 'n3w_Password'
        user = UserFactory.create()

        serializer = serializers.PasswordResetSerializer(user, data={
            'new_password': new_password,
            'new_password2': new_password,
        })
        self.assertTrue(serializer.is_valid())

        serializer.save()
        self.assertTrue(user.check_password(new_password))

    def test_deserialize_invalid_new_password(self):
        new_password = '2Short'
        user = UserFactory.build()

        serializer = serializers.PasswordResetSerializer(user, data={
            'new_password': new_password,
            'new_password2': new_password,
        })
        self.assertFalse(serializer.is_valid())
        self.assertIn('new_password', serializer.errors)

    def test_deserialize_mismatched_passwords(self):
        new_password = 'n3w_Password'
        other_password = 'other_new_password'
        user = UserFactory.create()

        serializer = serializers.PasswordResetSerializer(user, data={
            'new_password': new_password,
            'new_password2': other_password,
        })
        self.assertFalse(serializer.is_valid())


class RegistrationSerializerTest(TestCase):
    def setUp(self):
        super(RegistrationSerializerTest, self).setUp()
        self.data = {
            'name': "Robert'); DROP TABLE Students;--'",
            'email': 'Bobby.Tables+327@xkcd.com',
            'password': 'Sup3RSecre7paSSw0rD',
            'password2': 'Sup3RSecre7paSSw0rD',
        }

    def test_deserialize(self):
        serializer = serializers.RegistrationSerializer(data=self.data)
        self.assertTrue(serializer.is_valid())

        validated_data = serializer.validated_data
        self.assertEqual(validated_data['name'], self.data['name'])
        self.assertEqual(validated_data['email'], self.data['email'].lower())
        self.assertEqual(validated_data['password'], self.data['password'])

    def test_deserialize_invalid_new_password(self):
        self.data['password'] = '2short'

        serializer = serializers.RegistrationSerializer(data=self.data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('password', serializer.errors)

    def test_deserialize_mismatched_passwords(self):
        self.data['password2'] = 'different_password'
        serializer = serializers.RegistrationSerializer(data=self.data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('password2', serializer.errors)

    def test_deserialize_no_email(self):
        self.data['email'] = None

        serializer = serializers.RegistrationSerializer(data=self.data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('email', serializer.errors)


class UserSerializerTest(RequestTestCase):
    def test_serialize(self):
        user = UserFactory.create()
        request = self.create_request()
        context = {'request': request}
        serializer = serializers.UserSerializer(user, context=context)

        url = reverse(
            'user_management_api:user_detail',
            kwargs={'pk': user.pk},
            request=request,
        )

        expected = {
            'url': url,
            'name': user.name,
            'email': user.email,
            'date_joined': iso_8601(user.date_joined),
        }
        self.assertEqual(serializer.data, expected)

    def test_deserialize(self):
        user = UserFactory.build()
        data = {
            'name': 'New Name',
        }
        serializer = serializers.UserSerializer(user, data=data)
        self.assertTrue(serializer.is_valid())

    def test_deserialize_create_email_in_use(self):
        other_user = UserFactory.create()
        data = {
            'name': "Robert'); DROP TABLE Students;--'",
            'email': other_user.email,
            'password': 'Sup3RSecre7paSSw0rD',
            'password2': 'Sup3RSecre7paSSw0rD',
        }
        serializer = serializers.RegistrationSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertEqual(
            serializer._errors['email'],
            ['That email address has already been registered.'])

    def test_deserialize_update_email_in_use(self):
        user = UserFactory.create()
        other_user = UserFactory.create()
        data = {
            'name': "Robert'); DROP TABLE Students;--'",
            'email': other_user.email,
            'password': 'Sup3RSecre7paSSw0rD',
            'password2': 'Sup3RSecre7paSSw0rD',
        }
        serializer = serializers.RegistrationSerializer(user, data=data)
        self.assertFalse(serializer.is_valid())
        self.assertEqual(
            serializer._errors['email'],
            ['That email address has already been registered.'])


class TestUserSerializerCreate(TestCase):
    def test_deserialize_no_email(self):
        data = {
            'name': "Robert'); DROP TABLE Students;--'",
            'email': None,
            'password': 'Sup3RSecre7paSSw0rD',
            'password2': 'Sup3RSecre7paSSw0rD',
        }

        serializer = serializers.UserSerializerCreate(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('email', serializer.errors)

    def test_deserialize_email_in_use(self):
        other_user = UserFactory.create()
        data = {
            'name': "Robert'); DROP TABLE Students;--'",
            'email': other_user.email,
            'password': 'Sup3RSecre7paSSw0rD',
            'password2': 'Sup3RSecre7paSSw0rD',
        }
        serializer = serializers.UserSerializerCreate(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertEqual(
            serializer._errors['email'],
            ['That email address has already been registered.'],
        )


class SerializerPasswordsTest(TestCase):
    too_simple = (
        'Password must have at least ' +
        'one upper case letter, one lower case letter, and one number.'
    )

    too_fancy = (
        'Password only accepts the following symbols ' + string.punctuation
    )

    serializers = (
        (serializers.PasswordResetSerializer, 'new_password'),
        (serializers.PasswordChangeSerializer, 'new_password'),
        (serializers.RegistrationSerializer, 'password'),
    )

    def assert_validation_error(self, serializer_class, field, data, expected):
        serializer = serializer_class(data=data)
        serializer.is_valid()  # Perform validation
        on_missing = '{} not in {}.errors'.format(field, serializer)
        self.assertTrue(field in serializer.errors, on_missing)
        error = serializer.errors[field]
        on_wrong = 'Expected "{}", got "{}" on {}'.format(
            expected,
            error,
            serializer,
        )
        self.assertEqual(error, [expected], on_wrong)

    def assert_no_validation_error(self, serializer_class, field, data):
        serializer = serializer_class(data=data)
        serializer.is_valid()  # Perform validation
        on_present = '{} unexpectedly in {}.errors'.format(field, serializer)
        self.assertFalse(field in serializer.errors, on_present)

    def test_missing(self):
        data = {}
        for serializer_class, field in self.serializers:
            msg = Field.default_error_messages['required']
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_too_short(self):
        for serializer_class, field in self.serializers:
            data = {field: 'Aa1'}
            msg = 'Ensure this field has at least 8 characters.'
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_no_upper(self):
        for serializer_class, field in self.serializers:
            data = {field: 'aaaa1111'}
            msg = self.too_simple
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_no_lower(self):
        for serializer_class, field in self.serializers:
            data = {field: 'AAAA1111'}
            msg = self.too_simple
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_no_number(self):
        for serializer_class, field in self.serializers:
            data = {field: 'AAAAaaaa'}
            msg = self.too_simple
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_symbols(self):
        """Ensure all acceptable symbols are acceptable."""
        for serializer_class, field in self.serializers:
            for symbol in string.punctuation:
                data = {field: 'AAaa111' + symbol}
                self.assert_no_validation_error(serializer_class, field, data)

    def test_non_ascii(self):
        for serializer_class, field in self.serializers:
            data = {field: u'AA11aa££'}  # £ is not an ASCII character.
            msg = self.too_fancy
            self.assert_validation_error(serializer_class, field, data, msg)

    def test_ok(self):
        for serializer_class, field in self.serializers:
            data = {field: 'AAAaaa11'}
            self.assert_no_validation_error(serializer_class, field, data)


class ResendConfirmationEmailSerializerTest(TestCase):
    def test_serialize(self):
        """Assert user can request a new email confirmation."""
        user = UserFactory.create()
        data = {'email': user.email}
        serializer = serializers.ResendConfirmationEmailSerializer(data=data)
        self.assertTrue(serializer.is_valid(), msg=serializer.errors)

    def test_user_does_not_exist(self):
        """Assert user should exist before sending email confirmation."""
        data = {'email': 'a-non-existing@user.com'}
        serializer = serializers.ResendConfirmationEmailSerializer(data=data)
        self.assertFalse(serializer.is_valid())

    def test_user_already_validated(self):
        """Assert confirmation email is not send if user was already verified."""
        user = UserFactory.create(email_verified=True)
        data = {'email': user.email}
        serializer = serializers.ResendConfirmationEmailSerializer(data=data)
        self.assertFalse(serializer.is_valid())



from string import (
    ascii_letters,
    ascii_lowercase,
    ascii_uppercase,
    digits,
    punctuation
)

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _


too_simple = _(
    'Password must have at least ' +
    'one upper case letter, one lower case letter, and one number.'
)

too_fancy = _(
    'Password only accepts the following symbols !"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~'
)


def validate_password_strength(value):
    """
    Passwords should be tough.

    That means they should use:
    - mixed case letters,
    - numbers,
    - (optionally) ascii symbols and spaces.

    The (contrversial?) decision to limit the passwords to ASCII only
    is for the sake of:
    - simplicity (no need to normalise UTF-8 input)
    - security (some character sets are visible as typed into password fields)

    TODO: In future, it may be worth considering:
    - rejecting common passwords. (Where can we get a list?)
    - rejecting passwords with too many repeated characters.

    It should be noted that no restriction has been placed on the length of the
    password here, as that can easily be achieved with use of the min_length
    attribute of a form/serializer field.
    """
    used_chars = set(value)
    good_chars = set(ascii_letters + digits + punctuation + ' ')
    required_sets = (ascii_uppercase, ascii_lowercase, digits)

    if not used_chars.issubset(good_chars):
        raise ValidationError(too_fancy)

    for required in required_sets:
        if not used_chars.intersection(required):
            raise ValidationError(too_simple)



# -*- coding: utf-8 -*-

import unittest

import django
from django.contrib.sites.models import Site
from django.core import checks
from django.db.models import TextField
from django.db.utils import IntegrityError
from django.test import TestCase
from django.utils import six, timezone
from mock import Mock, patch

from user_management.models.tests import utils
from user_management.utils.notifications import validation_email_context
from . import models
from .factories import UserFactory
from .. import mixins


PASSWORD_CONTEXT = 'user_management.utils.notifications.password_reset_email_context'
VALIDATION_CONTEXT = 'user_management.utils.notifications.validation_email_context'
SEND_METHOD = 'user_management.utils.notifications.incuna_mail.send'

skip_if_checks_unavailable = unittest.skipIf(
    django.VERSION < (1, 7),
    'Checks only available in django>=1.7',
)


class TestUser(utils.APIRequestTestCase):
    """Test the "User" model"""
    model = models.User

    def test_fields(self):
        """Do we have the fields we expect?"""
        fields = self.model._meta.get_all_field_names()
        expected = {
            # On model
            'id',
            'name',
            'date_joined',
            'email',
            'email_verified',
            'is_active',
            'is_staff',
            'is_superuser',
            'last_login',
            'password',
            'avatar',

            # Incoming
            'groups',  # Django permission groups
            'user_permissions',
            'logentry',  # Django admin logs
            'authtoken',
            'auth_token',  # Rest framework authtoken
        }

        self.assertCountEqual(fields, expected)

    def test_str(self):
        """Does "User.__str__()" work as expected?"""
        expected = 'Leopold Stotch'
        user = self.model(name=expected)
        self.assertEqual(six.text_type(user), expected)

    def test_name_methods(self):
        """Do "User.get_full_name()" & "get_short_name()" work as expected?"""
        expected = 'Professor Chaos'
        user = self.model(name=expected)
        self.assertEqual(user.get_full_name(), expected)
        self.assertEqual(user.get_short_name(), expected)


class TestUserManager(TestCase):
    manager = models.User.objects

    def test_create_user_without_email(self):
        with self.assertRaises(ValueError):
            self.manager.create_user(email='')
        self.assertFalse(self.manager.count())

    def test_create_duplicate_email(self):
        existing_user = UserFactory.create()
        with self.assertRaises(IntegrityError):
            self.manager.create_user(email=existing_user.email.upper())

    def test_create_user(self):
        time_before = timezone.now()
        data = {
            'email': 'valid@example.com',
            'name': 'Mysterion',
            'password': 'I can N3ver DIE!'
        }

        # Call creation method of manager
        with self.assertNumQueries(1):
            # Only one query:
            #     INSERT INTO "users_user" ("fields",)
            #         VALUES ('blah') RETURNING "users_user"."id"
            result = self.manager.create_user(**data)

        # Check that user returned is the right one
        user = self.manager.get()
        self.assertEqual(user, result)

        # Check that the password is valid
        self.assertTrue(user.check_password(data['password']))

        # Check name/email is correct
        self.assertEqual(user.name, data['name'])
        self.assertEqual(user.email, data['email'])

        # Check defaults
        self.assertFalse(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_superuser)
        self.assertFalse(user.email_verified)

        # Check that the time is correct (or at least, in range)
        time_after = timezone.now()
        self.assertTrue(time_before < user.date_joined < time_after)

    def test_create_user_uppercase_email(self):
        email = 'VALID@EXAMPLE.COM'

        user = self.manager.create_user(email)
        self.assertEqual(email.lower(), user.email)

    def test_set_last_login(self):
        email = 'valid@example.com'

        before = timezone.now()
        user = self.manager.create_user(email)
        after = timezone.now()

        self.assertTrue(before < user.last_login < after)

    def test_create_superuser(self):
        email = 'valid@example.com'
        password = 'password'

        # Call creation method of manager:
        with self.assertNumQueries(1):
            # Only one query:
            #     INSERT INTO "users_user" ("fields",)
            #         VALUES ('blah') RETURNING "users_user"."id"
            result = self.manager.create_superuser(email, password)

        # Check that user returned is the right one
        user = self.manager.get()
        self.assertEqual(user, result)

        # Check that the password is valid
        self.assertTrue(user.check_password(password))

        # Check defaults
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertTrue(user.is_superuser)

    def test_get_by_natural_key(self):
        """Assert email is case-insensitive."""
        email = 'WHATDID@YOU.SAY'
        existing_user = UserFactory.create(email=email)

        user = self.manager.get_by_natural_key(email.lower())
        self.assertEqual(user, existing_user)


class TestVerifyEmailMixin(TestCase):
    model = models.VerifyEmailUser

    def test_save(self):
        user = self.model()
        user.save()
        self.assertFalse(user.is_active)
        self.assertFalse(user.email_verified)

    def test_email_context(self):
        """Assert `password_reset_email_context` returns the correct data."""
        mocked_user = Mock()
        mocked_site = Mock()

        class DummyNotification:
            user = mocked_user
            site = mocked_site

        notification = DummyNotification()
        context = validation_email_context(notification)

        expected_context = {
            'protocol': 'https',
            'token': mocked_user.generate_validation_token(),
            'site': mocked_site,
        }
        self.assertEqual(context, expected_context)

    def test_send_validation_email(self):
        context = {}
        site = Site.objects.get_current()
        user = self.model(email='email@email.email')

        with patch(VALIDATION_CONTEXT) as get_context:
            get_context.return_value = context
            with patch(SEND_METHOD) as send:
                user.send_validation_email()

        expected = {
            'to': user.email,
            'template_name': 'user_management/account_validation_email.txt',
            'html_template_name': 'user_management/account_validation_email.html',
            'subject': '{} account validate'.format(site.domain),
            'context': context
        }
        send.assert_called_once_with(**expected)

    def test_verified_email(self):
        user = self.model(email_verified=True)

        with patch(SEND_METHOD) as send:
            with self.assertRaises(ValueError):
                user.send_validation_email()

        self.assertFalse(send.called)

    @skip_if_checks_unavailable
    def test_manager_check_valid(self):
        errors = self.model.check()
        self.assertEqual(errors, [])

    @skip_if_checks_unavailable
    def test_manager_check_invalid(self):
        class InvalidUser(self.model):
            objects = mixins.UserManager()

        expected = [
            checks.Warning(
                "Manager should be an instance of 'VerifyEmailManager'",
                hint="Subclass a custom manager from 'VerifyEmailManager'",
                obj=InvalidUser,
                id='user_management.W001',
            ),
        ]
        errors = InvalidUser.check()
        self.assertEqual(errors, expected)


class TestCustomNameUser(utils.APIRequestTestCase):
    model = models.CustomNameUser

    def test_fields(self):
        """Do we have the fields we expect?"""
        fields = self.model._meta.get_all_field_names()
        expected = {
            # On model
            'id',
            'name',
            'date_joined',
            'email',
            'email_verified',
            'is_active',
            'is_staff',
            'last_login',
            'password',
            'avatar',
        }

        self.assertCountEqual(fields, expected)

    def test_name(self):
        expected = u'Cú Chulainn'
        model = self.model(name=expected)

        self.assertEqual(model.get_full_name(), expected)
        self.assertEqual(six.text_type(model), expected)
        field = self.model._meta.get_field_by_name('name')[0]
        self.assertIsInstance(field, TextField)

    @skip_if_checks_unavailable
    def test_manager_check_invalid(self):
        errors = self.model.check()
        self.assertEqual(errors, [])


class TestCustomPasswordResetNotification(TestCase):
    """Assert we can customise the notification to send a password reset."""
    model = models.CustomVerifyEmailUser

    def test_send_password_reset_email(self):
        """Assert `text_email_template` and `html_template_name` can be customised."""
        context = {}
        site = Site.objects.get_current()
        user = self.model(email='email@email.email')

        with patch(PASSWORD_CONTEXT) as get_context:
            get_context.return_value = context
            with patch(SEND_METHOD) as send:
                user.send_password_reset()

        expected = {
            'to': user.email,
            'template_name': 'my_custom_email.txt',
            'html_template_name': None,
            'subject': '{} password reset'.format(site.domain),
            'context': context
        }
        send.assert_called_once_with(**expected)



from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers, validators

from user_management.utils.validators import validate_password_strength


User = get_user_model()


class UniqueEmailValidator(validators.UniqueValidator):
    def filter_queryset(self, value, queryset):
        """Check lower-cased email is unique."""
        return super(UniqueEmailValidator, self).filter_queryset(
            value.lower(),
            queryset,
        )


unique_email_validator = UniqueEmailValidator(
    queryset=User.objects.all(),
    message=_('That email address has already been registered.'),
)


class ValidateEmailMixin(object):
    def validate_email(self, value):
        return value.lower()


class EmailSerializerBase(serializers.Serializer):
    """Serializer defining a read-only `email` field."""
    email = serializers.EmailField(max_length=511, label=_('Email address'))

    class Meta:
        fields = ['email']


class RegistrationSerializer(ValidateEmailMixin, serializers.ModelSerializer):
    email = serializers.EmailField(validators=[unique_email_validator])
    password = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('Password'),
        validators=[validate_password_strength],
    )
    password2 = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('Repeat password'),
    )

    class Meta:
        fields = ['name', 'email', 'password', 'password2']
        model = User

    def validate(self, attrs):
        password2 = attrs.pop('password2')
        if password2 != attrs.get('password'):
            msg = _('Your passwords do not match.')
            raise serializers.ValidationError({'password2': msg})
        return attrs

    def create(self, validated_data):
        password = validated_data.pop('password')
        user = self.Meta.model.objects.create(**validated_data)
        user.set_password(password)
        user.save()
        return user


class PasswordChangeSerializer(serializers.ModelSerializer):
    old_password = serializers.CharField(
        write_only=True,
        label=_('Old password'),
    )
    new_password = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('New password'),
        validators=[validate_password_strength],
    )
    new_password2 = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('Repeat new password'),
    )

    class Meta:
        model = User
        fields = ('old_password', 'new_password', 'new_password2')

    def update(self, instance, validated_data):
        """Check the old password is valid and set the new password."""
        if not instance.check_password(validated_data['old_password']):
            msg = _('Invalid password.')
            raise serializers.ValidationError({'old_password': msg})

        instance.set_password(validated_data['new_password'])
        instance.save()
        return instance

    def validate(self, attrs):
        if attrs.get('new_password') != attrs['new_password2']:
            msg = _('Your new passwords do not match.')
            raise serializers.ValidationError({'new_password2': msg})
        if attrs.get('old_password') == attrs.get('new_password'):
            msg = _('Your password has not changed.')
            raise serializers.ValidationError({'new_password': msg})
        return attrs


class PasswordResetSerializer(serializers.ModelSerializer):
    new_password = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('New password'),
        validators=[validate_password_strength],
    )
    new_password2 = serializers.CharField(
        write_only=True,
        min_length=8,
        label=_('Repeat new password'),
    )

    class Meta:
        model = User
        fields = ('new_password', 'new_password2')

    def update(self, instance, validated_data):
        """Set the new password for the user."""
        instance.set_password(validated_data['new_password'])
        instance.save()
        return instance

    def validate(self, attrs):
        if attrs.get('new_password') != attrs['new_password2']:
            msg = _('Your new passwords do not match.')
            raise serializers.ValidationError({'new_password2': msg})
        return attrs


class PasswordResetEmailSerializer(EmailSerializerBase):
    """Serializer defining an `email` field to reset password."""


class ProfileSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('name', 'email', 'date_joined')
        read_only_fields = ('email', 'date_joined')


class ResendConfirmationEmailSerializer(EmailSerializerBase):
    """Serializer defining an `email` field to resend a confirmation email."""
    def validate_email(self, email):
        """
        Validate if email exists and requires a verification.

        `validate_email` will set a `user` attribute on the instance allowing
        the view to send an email confirmation.
        """
        try:
            self.user = User.objects.get_by_natural_key(email)
        except User.DoesNotExist:
            msg = _('A user with this email address does not exist.')
            raise serializers.ValidationError(msg)

        if self.user.email_verified:
            msg = _('User email address is already verified.')
            raise serializers.ValidationError(msg)
        return email


class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = ('url', 'name', 'email', 'date_joined')
        read_only_fields = ('email', 'date_joined')
        view_name = 'user_management_api:user_detail'


class UserSerializerCreate(ValidateEmailMixin, UserSerializer):
    email = serializers.EmailField(validators=[unique_email_validator])

    class Meta(UserSerializer.Meta):
        read_only_fields = ('date_joined',)



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt

from .. import views


urlpatterns = patterns(
    '',
    url(
        regex=(
            r'^verify_email/(?P<token>[0-9A-Za-z:\-_]+)/?$'
        ),
        view=csrf_exempt(views.VerifyAccountView.as_view()),
        name='verify_user',
    ),
)



import binascii
import datetime
import os

from django.conf import settings
from django.db import models
from django.utils import timezone


MINUTE = 60
HOUR = 60 * MINUTE
DAY = 24 * HOUR
# Max expiry time for auth tokens
DEFAULT_AUTH_TOKEN_MAX_AGE = 200 * DAY
# Max inactivity time
DEFAULT_AUTH_TOKEN_MAX_INACTIVITY = 12 * HOUR


def update_expiry(created=None):
    now = timezone.now()

    if created is None:
        created = now

    max_age = getattr(
        settings,
        'AUTH_TOKEN_MAX_AGE',
        DEFAULT_AUTH_TOKEN_MAX_AGE,
    )
    max_age = created + datetime.timedelta(seconds=max_age)

    max_inactivity = getattr(
        settings,
        'AUTH_TOKEN_MAX_INACTIVITY',
        DEFAULT_AUTH_TOKEN_MAX_INACTIVITY,
    )
    max_inactivity = now + datetime.timedelta(seconds=max_inactivity)

    return min(max_inactivity, max_age)


class AuthToken(models.Model):
    """
    Model for auth tokens with added functionality of controlling
    expiration time of tokens.

    Similar to DRF's model but with extra `expires` field.

    It also has FK (not OneToOne relation) to user as the user can have
    many tokens (multiple devices) in order to token expiration to work.
    """
    key = models.CharField(max_length=40, primary_key=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='authtoken')
    created = models.DateTimeField(default=timezone.now, editable=False)
    expires = models.DateTimeField(default=update_expiry, editable=False)

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(AuthToken, self).save(*args, **kwargs)

    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()

    def update_expiry(self, commit=True):
        """Update token's expiration datetime on every auth action."""
        self.expires = update_expiry(self.created)
        if commit:
            self.save()



import factory

from django.contrib.auth import get_user_model

from user_management.api.models import AuthToken


class UserFactory(factory.DjangoModelFactory):
    name = factory.Sequence('Test User {}'.format)
    email = factory.Sequence('email{}@example.com'.format)
    is_active = True

    class Meta:
        model = get_user_model()

    @factory.post_generation
    def password(self, create, extracted='default password', **kwargs):
        self.raw_password = extracted
        self.set_password(self.raw_password)
        if create:
            self.save()


class AuthTokenFactory(factory.DjangoModelFactory):
    key = factory.Sequence('key{}'.format)
    user = factory.SubFactory(UserFactory)

    class Meta:
        model = AuthToken



from django.conf.urls import include, url


urlpatterns = [
    url(r'', include(
        [
            url(r'', include('user_management.api.urls')),
            url(r'', include('user_management.api.urls.verify_email')),
            url(r'', include('user_management.api.urls.users')),
            url(r'', include('user_management.api.avatar.urls')),
        ],
        namespace='user_management_api'
    ))
]



import datetime

import mock
from django.http import QueryDict
from django.test import TestCase
from django.utils import timezone
from rest_framework import exceptions

from user_management.models.tests.factories import AuthTokenFactory, UserFactory
from ..authentication import FormTokenAuthentication, TokenAuthentication


class TestFormTokenAuthentication(TestCase):
    def test_no_token(self):
        request = mock.Mock(DATA=QueryDict(''))
        response = FormTokenAuthentication().authenticate(request)
        self.assertIsNone(response)

    def test_invalid_token(self):
        data = QueryDict('', mutable=True)
        data.update({'token': 'WOOT'})
        request = mock.Mock(DATA=data)
        response = FormTokenAuthentication().authenticate(request)
        self.assertIsNone(response)

    def test_valid_token(self):
        token = AuthTokenFactory.create()
        data = QueryDict('', mutable=True)
        data.update({'token': token.key})
        request = mock.Mock(DATA=data)
        response = FormTokenAuthentication().authenticate(request)
        expected = (token.user, token)
        self.assertEqual(response, expected)


class TestTokenAuthentication(TestCase):
    def setUp(self):
        self.now = timezone.now()
        self.days = 1
        self.key = 'k$y'
        self.user = UserFactory.create()
        self.auth = TokenAuthentication()

    def _create_token(self, when):
        token = AuthTokenFactory.create(
            key=self.key,
            user=self.user,
            expires=when,
        )

        return token

    def test_token_expiry_if_valid(self):
        # Token is valid till tomorrow
        tomorrow = self.now + datetime.timedelta(days=self.days)
        token_old = self._create_token(when=tomorrow)

        user, token = self.auth.authenticate_credentials(self.key)

        self.assertEqual(token, token_old)
        self.assertEqual(user, self.user)

    def test_token_expiry_if_has_expired(self):
        # Token has expired yesterday
        yesterday = self.now - datetime.timedelta(days=self.days)
        self._create_token(when=yesterday)

        with self.assertRaises(exceptions.AuthenticationFailed):
            self.auth.authenticate_credentials(self.key)

    def test_token_inactivity(self):
        # Create token valid till tomorrow
        tomorrow = self.now + datetime.timedelta(days=self.days)
        token_example = self._create_token(when=tomorrow)

        user, token = self.auth.authenticate_credentials(self.key)

        # Ensure tokens are correct
        self.assertEqual(token, token_example)

        # Auth again with very low inactivity time
        # Token's expiry gets updated in auth process
        with self.settings(AUTH_TOKEN_MAX_INACTIVITY=0):
            self.auth.authenticate_credentials(self.key)

        # User's token has expired now
        with self.assertRaises(exceptions.AuthenticationFailed):
            self.auth.authenticate_credentials(self.key)



from django.utils.translation import ugettext_lazy as _
from rest_framework import status
from rest_framework.exceptions import APIException


class InvalidExpiredToken(APIException):
    """Exception to confirm an account."""
    status_code = status.HTTP_400_BAD_REQUEST
    default_detail = _('Invalid or expired token.')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import user_management.api.models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AuthToken',
            fields=[
                ('key', models.CharField(primary_key=True, serialize=False, max_length=40)),
                ('created', models.DateTimeField(editable=False, default=django.utils.timezone.now)),
                ('expires', models.DateTimeField(editable=False, default=user_management.api.models.update_expiry)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='authtoken',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='authtoken'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import user_management.models.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(verbose_name='password', max_length=128)),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('is_superuser', models.BooleanField(help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status', default=False)),
                ('date_joined', models.DateTimeField(editable=False, verbose_name='date joined', default=django.utils.timezone.now)),
                ('email', models.EmailField(unique=True, verbose_name='Email address', max_length=511)),
                ('is_staff', models.BooleanField(verbose_name='staff status', default=False)),
                ('name', models.CharField(verbose_name='Name', max_length=255)),
                ('is_active', models.BooleanField(verbose_name='active', default=False)),
                ('email_verified', models.BooleanField(help_text='Indicates if the email address has been verified.', verbose_name='Email verified?', default=False)),
                ('avatar', models.ImageField(null=True, blank=True, upload_to='user_avatar')),
                ('groups', models.ManyToManyField(help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_query_name='user', blank=True, related_name='user_set', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(help_text='Specific permissions for this user.', related_query_name='user', blank=True, related_name='user_set', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
            bases=(user_management.models.mixins.EmailVerifyUserMethodsMixin, user_management.models.mixins.NameUserMethodsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BasicUser',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(verbose_name='password', max_length=128)),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('date_joined', models.DateTimeField(editable=False, verbose_name='date joined', default=django.utils.timezone.now)),
                ('email', models.EmailField(unique=True, verbose_name='Email address', max_length=511)),
                ('is_staff', models.BooleanField(verbose_name='staff status', default=False)),
                ('name', models.CharField(verbose_name='Name', max_length=255)),
            ],
            options={
                'abstract': False,
            },
            bases=(user_management.models.mixins.NameUserMethodsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CustomNameUser',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(verbose_name='password', max_length=128)),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('date_joined', models.DateTimeField(editable=False, verbose_name='date joined', default=django.utils.timezone.now)),
                ('email', models.EmailField(unique=True, verbose_name='Email address', max_length=511)),
                ('is_staff', models.BooleanField(verbose_name='staff status', default=False)),
                ('is_active', models.BooleanField(verbose_name='active', default=False)),
                ('email_verified', models.BooleanField(help_text='Indicates if the email address has been verified.', verbose_name='Email verified?', default=False)),
                ('avatar', models.ImageField(null=True, blank=True, upload_to='user_avatar')),
                ('name', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=(user_management.models.mixins.EmailVerifyUserMethodsMixin, user_management.models.mixins.NameUserMethodsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CustomVerifyEmailUser',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(verbose_name='password', max_length=128)),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('date_joined', models.DateTimeField(editable=False, verbose_name='date joined', default=django.utils.timezone.now)),
                ('email', models.EmailField(unique=True, verbose_name='Email address', max_length=511)),
                ('is_staff', models.BooleanField(verbose_name='staff status', default=False)),
                ('name', models.CharField(verbose_name='Name', max_length=255)),
                ('is_active', models.BooleanField(verbose_name='active', default=False)),
                ('email_verified', models.BooleanField(help_text='Indicates if the email address has been verified.', verbose_name='Email verified?', default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(user_management.models.mixins.EmailVerifyUserMethodsMixin, user_management.models.mixins.NameUserMethodsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='VerifyEmailUser',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(verbose_name='password', max_length=128)),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('date_joined', models.DateTimeField(editable=False, verbose_name='date joined', default=django.utils.timezone.now)),
                ('email', models.EmailField(unique=True, verbose_name='Email address', max_length=511)),
                ('is_staff', models.BooleanField(verbose_name='staff status', default=False)),
                ('name', models.CharField(verbose_name='Name', max_length=255)),
                ('is_active', models.BooleanField(verbose_name='active', default=False)),
                ('email_verified', models.BooleanField(help_text='Indicates if the email address has been verified.', verbose_name='Email verified?', default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(user_management.models.mixins.EmailVerifyUserMethodsMixin, user_management.models.mixins.NameUserMethodsMixin, models.Model),
        ),
    ]



from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from rest_framework import authentication, exceptions
from rest_framework.authentication import TokenAuthentication as DRFTokenAuthentication

from .models import AuthToken


class FormTokenAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        """
        Authenticate a user from a token form field

        Errors thrown here will be swallowed by django-rest-framework, and it
        expects us to return None if authentication fails.
        """
        try:
            key = request.DATA['token']
        except KeyError:
            return

        try:
            token = AuthToken.objects.get(key=key)
        except AuthToken.DoesNotExist:
            return

        return (token.user, token)


class TokenAuthentication(DRFTokenAuthentication):
    model = AuthToken

    def authenticate_credentials(self, key):
        """Custom authentication to check if auth token has expired."""
        user, token = super(TokenAuthentication, self).authenticate_credentials(key)

        if token.expires < timezone.now():
            msg = _('Token has expired.')
            raise exceptions.AuthenticationFailed(msg)

        # Update the token's expiration date
        token.update_expiry()

        return (user, token)



from raven.contrib.django.models import get_client
from rest_framework.test import APIRequestFactory

from user_management.models.tests.utils import APIRequestTestCase


class TestSentryClient(APIRequestTestCase):
    def test_cookies(self):
        request = APIRequestFactory().post('/')
        request.COOKIES['username'] = 'jimmy'

        raven = get_client()
        result = raven.get_data_from_request(request)

        self.assertFalse('cookies' in result['request'])
        self.assertFalse('Cookie' in result['request']['headers'])



import datetime

import mock
from django.test.utils import override_settings
from django.utils import timezone

from user_management.api.models import AuthToken
from user_management.models.tests import utils
from user_management.models.tests.factories import AuthTokenFactory
from ..management.commands import remove_expired_tokens


class TestRemoveExpiredTokensManagementCommand(utils.APIRequestTestCase):
    def setUp(self):
        self.command = remove_expired_tokens.Command()
        self.command.stdout = mock.MagicMock()

    def test_no_tokens_removed(self):
        """Tests that non-expired tokens are not removed."""
        tomorrow = timezone.now() + datetime.timedelta(days=1)
        token = AuthTokenFactory.create(expires=tomorrow)

        self.command.handle()

        expected = AuthToken.objects.all()
        self.assertCountEqual(expected, [token])

    @override_settings(AUTH_TOKEN_MAX_EXPIRY=7)
    def test_expired_tokens(self):
        """Ensure expired token is removed from db while valid one remains."""
        now = timezone.now()
        tomorrow = now + datetime.timedelta(days=1)
        valid_token = AuthTokenFactory.create(expires=tomorrow)

        # this token is expired
        long_ago = now - datetime.timedelta(days=33)
        AuthTokenFactory.create(expires=long_ago)

        self.command.handle()
        expected = AuthToken.objects.all()

        self.assertCountEqual(expected, [valid_token])



from user_management.utils.notifications import PasswordResetNotification


class CustomPasswordResetNotification(PasswordResetNotification):
    """Test setting a custom notification to alter how we send the password reset."""
    text_email_template = 'my_custom_email.txt'
    html_email_template = None



def iso_8601(datetime):
    """Convert a datetime into an iso 8601 string."""
    if datetime is None:
        return datetime
    value = datetime.isoformat()
    if value.endswith('+00:00'):
        value = value[:-6] + 'Z'
    return value
